"""Utilities for building composable CLIs from command dataclasses."""

from byteb4rb1e.utils.argparse.command import CLICommand
from byteb4rb1e.utils.argparse.dispatcher import CLI

__all__ = ["CLI", "CLICommand"]
